!-----------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations         !
!   Copyright (C) 2006 - 2008  CP2K developers group                          !
!-----------------------------------------------------------------------------!

! *****************************************************************************
!> \par History
!>      JGH 23-Jan-2006 : initial version
!>      Adapted for new interface
!>      IAB 09-Jan-2009 : Modified to cache plans in fft_plan_type
!>                        (c) The Numerical Algorithms Group (NAG) Ltd, 2009 on behalf of the HECToR project
!> \author JGH
! *****************************************************************************
SUBROUTINE fftw3_get_lengths ( DATA, max_length )

  IMPLICIT NONE 

  INTEGER, DIMENSION(*)                    :: DATA
  INTEGER, INTENT(INOUT)                   :: max_length

  INTEGER, DIMENSION(:), ALLOCATABLE       :: dlocal, idx
  INTEGER                                  :: h, i, j, k, m, maxn, &
                                              maxn_elevens, maxn_fives, &
                                              maxn_sevens, maxn_thirteens, &
                                              maxn_threes, maxn_twos, &
                                              ndata, nmax, number

!------------------------------------------------------------------------------
! compute ndata
!! FFTW can do arbitrary(?) lenghts, maybe you want to limit them to some
!!    powers of small prime numbers though...

  maxn_twos = 15
  maxn_threes = 3
  maxn_fives = 2
  maxn_sevens = 1
  maxn_elevens = 1
  maxn_thirteens = 0
  maxn = 37748736

  ndata = 0
  DO h = 0, maxn_twos
     nmax = HUGE(0) / 2**h
     DO i = 0, maxn_threes
        DO j = 0, maxn_fives
           DO k = 0, maxn_sevens
              DO m = 0, maxn_elevens
                 number = (3**i) * (5**j) * (7**k) * (11**m)

                 IF ( number > nmax ) CYCLE

                 number = number * 2 ** h
                 IF ( number >= maxn ) CYCLE

                 ndata = ndata + 1
              END DO
           END DO
        END DO
     END DO
  END DO

  ALLOCATE ( dlocal ( ndata ), idx ( ndata ) )

  ndata = 0
  dlocal ( : ) = 0
  DO h = 0, maxn_twos
     nmax = HUGE(0) / 2**h
     DO i = 0, maxn_threes
        DO j = 0, maxn_fives
           DO k = 0, maxn_sevens
              DO m = 0, maxn_elevens
                 number = (3**i) * (5**j) * (7**k) * (11**m)

                 IF ( number > nmax ) CYCLE

                 number = number * 2 ** h
                 IF ( number >= maxn ) CYCLE

                 ndata = ndata + 1
                 dlocal ( ndata ) = number
              END DO
           END DO
        END DO
     END DO
  END DO

  CALL sortint ( dlocal, ndata, idx )
  ndata = MIN ( ndata, max_length )
  DATA(1:ndata) = dlocal(1:ndata)
  max_length = ndata

  DEALLOCATE ( dlocal, idx )

END SUBROUTINE fftw3_get_lengths

! *****************************************************************************

SUBROUTINE fftw3_create_plan_3d(plan, zin, zout, plan_style, aligned)

  USE fft_plan,                           ONLY : fft_plan_type, dp

  IMPLICIT NONE

  TYPE(fft_plan_type), INTENT ( INOUT )              :: plan
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(IN)         :: zin
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(IN)         :: zout
  INTEGER                                            :: plan_style
  LOGICAL                                            :: aligned
  INTEGER                                            :: n1,n2,n3

#if defined ( __FFTW3 )
#include "fftw3.f"
  INTEGER :: fftw_plan_type
  SELECT CASE(plan_style)
  CASE(1)
         fftw_plan_type = FFTW_ESTIMATE 
  CASE(2)
         fftw_plan_type = FFTW_MEASURE
  CASE(3)
         fftw_plan_type = FFTW_PATIENT
  CASE(4)
         fftw_plan_type = FFTW_EXHAUSTIVE
  CASE DEFAULT
   STOP "fftw3_create_plan_3d"
  END SELECT

  IF (aligned) THEN
    fftw_plan_type = fftw_plan_type + FFTW_UNALIGNED
  END IF

  n1 = plan%n_3d(1)
  n2 = plan%n_3d(2)
  n3 = plan%n_3d(3)

#if defined(__SGL) || defined(__FFTSGL)
  IF ( plan%fft_in_place .AND. plan%fsign == +1 ) THEN
    CALL sfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zin,FFTW_FORWARD,fftw_plan_type)
  ELSE IF ( plan%fft_in_place .AND. plan%fsign == -1 ) THEN
    CALL sfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zin,FFTW_BACKWARD,fftw_plan_type)
  ELSE IF ( .NOT. plan%fft_in_place .AND. plan%fsign == +1 ) THEN
    CALL sfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zout,FFTW_FORWARD,fftw_plan_type)
  ELSE IF ( .NOT. plan%fft_in_place .AND. plan%fsign == -1 ) THEN
    CALL sfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zout,FFTW_BACKWARD,fftw_plan_type)
  END IF
#else
  IF ( plan%fft_in_place .AND. plan%fsign == +1 ) THEN
    CALL dfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zin,FFTW_FORWARD,fftw_plan_type)
  ELSE IF ( plan%fft_in_place .AND. plan%fsign == -1 ) THEN
    CALL dfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zin,FFTW_BACKWARD,fftw_plan_type)
  ELSE IF ( .NOT. plan%fft_in_place .AND. plan%fsign == +1 ) THEN
    CALL dfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zout,FFTW_FORWARD,fftw_plan_type)
  ELSE IF ( .NOT. plan%fft_in_place .AND. plan%fsign == -1 ) THEN
    CALL dfftw_plan_dft_3d(plan%fftw_plan,n1,n2,n3,zin,zout,FFTW_BACKWARD,fftw_plan_type)
  END IF
#endif

#endif

END SUBROUTINE fftw3_create_plan_3d

SUBROUTINE fftw33d ( plan, scale, zin, zout, stat )

  USE fft_plan,                           ONLY : fft_plan_type, dp

  IMPLICIT NONE

  TYPE(fft_plan_type), INTENT(IN)                      :: plan
  REAL(KIND=dp), INTENT(IN)                            :: scale
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(INOUT)        :: zin
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(INOUT)        :: zout
  INTEGER, INTENT(OUT)                                 :: stat

  INTEGER                                  :: n1, n2, n3

!------------------------------------------------------------------------------

  n1 = plan%n_3d(1)
  n2 = plan%n_3d(2)
  n3 = plan%n_3d(3)

#if defined ( __FFTW3 )
  stat = 1

#if defined(__SGL) || defined(__FFTSGL)
  IF ( plan%fft_in_place ) THEN
    CALL sfftw_execute_dft(plan%fftw_plan, zin, zin)
  ELSE
    CALL sfftw_execute_dft(plan%fftw_plan, zin, zout)
  END IF
#else
  IF ( plan%fft_in_place ) THEN
    CALL dfftw_execute_dft(plan%fftw_plan, zin, zin)
  ELSE
    CALL dfftw_execute_dft(plan%fftw_plan, zin, zout)
  END IF
#endif

  IF ( scale /= 1.0_dp ) THEN
#if defined(__SGL) || defined(__FFTSGL)
    IF ( plan%fft_in_place ) THEN
      CALL csscal(n1*n2*n3,scale,zin,1)
    ELSE
      CALL csscal(n1*n2*n3,scale,zout,1)
    END IF
#else
    IF ( plan%fft_in_place ) THEN
      CALL zdscal(n1*n2*n3,scale,zin,1)
    ELSE
      CALL zdscal(n1*n2*n3,scale,zout,1)
    END IF
#endif
  END IF

#else

  stat = 0

#endif

END SUBROUTINE fftw33d

! *****************************************************************************

SUBROUTINE fftw3_create_plan_1dm(plan, zin, zout, plan_style, aligned)

  USE fft_plan,                           ONLY : fft_plan_type, dp

  TYPE(fft_plan_type), INTENT ( INOUT )              :: plan
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(IN)         :: zin
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(IN)         :: zout
  INTEGER                                            :: plan_style
  LOGICAL                                            :: aligned
  INTEGER                                            :: ii,di,io,DO

#if defined ( __FFTW3 )
#include "fftw3.f"
  INTEGER :: fftw_plan_type 
  SELECT CASE(plan_style)
  CASE(1)
         fftw_plan_type = FFTW_ESTIMATE
  CASE(2)
         fftw_plan_type = FFTW_MEASURE
  CASE(3)
         fftw_plan_type = FFTW_PATIENT
  CASE(4)
         fftw_plan_type = FFTW_EXHAUSTIVE
  CASE DEFAULT
   STOP "fftw3_create_plan_1dm"
  END SELECT

  IF (aligned) THEN
    fftw_plan_type = fftw_plan_type + FFTW_UNALIGNED
  END IF

  ii = 1
  di = plan%n
  io = 1
  DO = plan%n
  IF ( plan%fsign == +1 .AND. plan%trans ) THEN
    ii = plan%m
    di = 1
  ELSEIF ( plan%fsign == -1 .AND. plan%trans ) THEN
    io = plan%m
    DO = 1
  END IF

#if defined(__SGL) || defined(__FFTSGL)
  IF ( plan%fsign == +1 ) THEN
    CALL sfftw_plan_many_dft(plan%fftw_plan,1,plan%n,plan%m,zin,0,ii,di,&
              zout,0,io,DO,FFTW_FORWARD,fftw_plan_type)
  ELSE
    CALL sfftw_plan_many_dft(plan%fftw_plan,1,plan%n,plan%m,zin,0,ii,di,&
              zout,0,io,DO,FFTW_BACKWARD,fftw_plan_type)
  END IF
#else
  IF ( plan%fsign == +1 ) THEN
    CALL dfftw_plan_many_dft(plan%fftw_plan,1,plan%n,plan%m,zin,0,ii,di,&
              zout,0,io,DO,FFTW_FORWARD,fftw_plan_type)
  ELSE
    CALL dfftw_plan_many_dft(plan%fftw_plan,1,plan%n,plan%m,zin,0,ii,di,&
              zout,0,io,DO,FFTW_BACKWARD,fftw_plan_type)
  END IF
#endif

#endif

END SUBROUTINE fftw3_create_plan_1dm

SUBROUTINE fftw3_destroy_plan ( plan )

  USE fft_plan,                           ONLY: fft_plan_type

  IMPLICIT NONE

  TYPE(fft_plan_type), INTENT (INOUT)   :: plan

#if defined ( __FFTW3 )
#if defined(__SGL) || defined(__FFTSGL)
  CALL sfftw_destroy_plan(plan%fftw_plan)
#else
  CALL dfftw_destroy_plan(plan%fftw_plan)
#endif
#endif

END SUBROUTINE fftw3_destroy_plan

SUBROUTINE fftw31dm ( plan, zin, zout, scale, stat )

  USE fft_plan,                 ONLY : fft_plan_type, dp

  IMPLICIT NONE
  TYPE (fft_plan_type), INTENT (IN)                  :: plan
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(IN)         :: zin
  COMPLEX(KIND=dp), DIMENSION(*), INTENT(INOUT)      :: zout
  REAL (KIND=dp), INTENT ( IN )                      :: scale
  INTEGER, INTENT(OUT)                               :: stat

!------------------------------------------------------------------------------

#if defined ( __FFTW3 )
  stat=1
#if defined(__SGL) || defined(__FFTSGL)
  CALL sfftw_execute_dft(plan%fftw_plan, zin, zout)
  IF ( scale /= 1.0_dp ) CALL csscal(plan%n*plan%m,scale,zout,1)
#else
  CALL dfftw_execute_dft(plan%fftw_plan, zin, zout)
  IF ( scale /= 1.0_dp ) CALL zdscal(plan%n*plan%m,scale,zout,1)
#endif
#else

  stat=0

#endif

END SUBROUTINE fftw31dm

!     Copyright (c) 2003, 2006 Matteo Frigo
!     Copyright (c) 2003, 2006 Massachusetts Institute of Technology
!     
!     This program is free software; you can redistribute it and/or modify
!     it under the terms of the GNU General Public License as published by
!     the Free Software Foundation; either version 2 of the License, or
!     (at your option) any later version.
!     
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!     
!     You should have received a copy of the GNU General Public License
!     along with this program; if not, write to the Free Software
!     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     
!     This is an example implementation of Fortran wisdom export/import
!     to/from a Fortran unit (file), exploiting the generic
!     dfftw_export_wisdom/dfftw_import_wisdom functions.
!     
!     We cannot compile this file into the FFTW library itself, lest all
!     FFTW-calling programs be required to link to the Fortran I/O
!     libraries.
!
!     adapted to become more standard Fortran 90 [2007-10] Joost VandeVondele
!     and added some namespacing
!     
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! *****************************************************************************
      SUBROUTINE fftw_write_char(c, iunit)
         IMPLICIT NONE
         CHARACTER c
         INTEGER iunit
         WRITE(iunit,'(a)',ADVANCE="NO") c
      END      

! *****************************************************************************
      SUBROUTINE fftw_export_wisdom_to_file(iunit)
         IMPLICIT NONE
         INTEGER iunit
         EXTERNAL fftw_write_char
#if defined ( __FFTW3 )
#if defined (__SGL) || defined(__FFTSGL)
         CALL sfftw_export_wisdom(fftw_write_char, iunit)
#else
         CALL dfftw_export_wisdom(fftw_write_char, iunit)
#endif
#endif
      END

!     Fortran 77 does not have any portable way to read an arbitrary
!     file one character at a time [needs to wait for stream IO of F2003].
!     The best alternative seems to be to
!     read a whole line into a buffer, since for fftw-exported wisdom we
!     can bound the line length.  (If the file contains longer lines,
!     then the lines will be truncated and the wisdom import should
!     simply fail.)  Ugh (and not thread safe).

! *****************************************************************************
      SUBROUTINE fftw_read_char(ic, iunit)
         IMPLICIT NONE
         INTEGER ic
         INTEGER iunit
         CHARACTER*256 buf
         SAVE buf
         INTEGER ibuf
         DATA ibuf/257/
         SAVE ibuf
         IF (ibuf .LT. 257) THEN
            ic = ICHAR(buf(ibuf:ibuf))
            ibuf = ibuf + 1
            RETURN
         ENDIF
         READ(iunit,123,END=666) buf
         ic = ICHAR(buf(1:1))
         ibuf = 2
         RETURN
 666     ic = -1
         ibuf = 257
 123     FORMAT(a256)
      END
      
! *****************************************************************************
      SUBROUTINE fftw_import_wisdom_from_file(isuccess, iunit)
         IMPLICIT NONE
         INTEGER isuccess
         INTEGER iunit
         EXTERNAL fftw_read_char
         isuccess=0
#if defined ( __FFTW3 )
#if defined (__SGL) || defined(__FFTSGL)
         CALL sfftw_import_wisdom(isuccess, fftw_read_char, iunit)
#else
         CALL dfftw_import_wisdom(isuccess, fftw_read_char, iunit)
#endif
#endif
      END
